import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.FactoryConfigurationError;  
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;  
import org.xml.sax.SAXParseException;
import java.io.*;
import org.w3c.dom.Document;
import org.w3c.dom.DOMException;

import java.util.*;
 


public class XML2T4CSV
{

    static Document document;
    HashMap hmNames = new HashMap();
    File outTimeLineFile = new File( "xml2t4csv.csv" );
    PrintWriter	outTimeLinePrint;
    boolean verboseStatus = true;

	public XML2T4CSV()
	{
	    try {
		outTimeLinePrint = new PrintWriter( new FileOutputStream( outTimeLineFile ), true );
	    } catch (FileNotFoundException fnfe) {
		System.out.println( "Exception: " + fnfe );
		System.exit( -1 );
	    }


	} // Constructor
 
    public static void main(String argv[])
    {
        if (argv.length != 1) {
            System.err.println("Usage: java XML2T4CSV filename");
            System.exit(1);
        }

	XML2T4CSV xml2t4 = new XML2T4CSV();

	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	// factory.setValidating(true);
    	// factory.setNamespaceAware(true);

    	try {

		DocumentBuilder builder = factory.newDocumentBuilder();
		document = builder.parse( new File(argv[0]) );
				
  		builder.setErrorHandler(
			new org.xml.sax.ErrorHandler() {    
				public void fatalError(SAXParseException exception)
							throws SAXException {
				}

				// treat validation errors as fatal
				public void error(SAXParseException e)
							throws SAXParseException
				{
				    throw e;
				}

      				// dump warnings too
      				public void warning(SAXParseException err)
      							throws SAXParseException
      				{
				    System.out.println("** Warning"
           				+ ", line " + err.getLineNumber()
           				+ ", uri " + err.getSystemId());
        				System.out.println("   " + err.getMessage());
      				}
  			}
		); 
	 

		// Go through document tree to create column names for T4 output file
		AdapterNode rootNode = new AdapterNode( document );
		AdapterNode currNode = new AdapterNode( document );
		AdapterNode obsNode = new AdapterNode( document );
		AdapterNode wrkNode = new AdapterNode( document );
		boolean endOfObservations = false;
		StringBuffer pfxStr = new StringBuffer("***Not Used Yet***");
		StringBuffer keyStr = new StringBuffer("***Not Used Yet***");
		while( !endOfObservations ) {

		    // From the root node, find the WEBLOGICMONITORSESSION element
		    boolean elementFnd = false;
		    while( !elementFnd ) {
			
			if ( currNode.domNode.hasChildNodes() ) {

			    for( int i = 0; i < currNode.domNode.getChildNodes().getLength(); i++ ) {
				
				if ( currNode.domNode.getChildNodes().item( i ).getNodeName().equals(
								    "WEBLOGICMONITORSESSION") ) {
				    if (xml2t4.verboseStatus ) {
					System.out.println( "Found: " + 
							currNode.domNode.getChildNodes().item( i ).getNodeName() );
				    }
				
				    elementFnd = true;
				    currNode = new AdapterNode( currNode.domNode.getChildNodes().item( i ) );

				}
			       
			    } // endfor

			} else {
			    System.out.println("ERROR: WEBLOGICMONITORSESSION element not found!");
			    System.exit( -1 );
			}
		    } 
		    // Assumption is that WEBLOGICMONITORSESSION element found
		    // Therefore, we look through it's child nodes for all OBSERVATION elements
		    // so that we can build the necessary column names
		    if ( currNode.domNode.hasChildNodes() ) {
			int lmtChildren = currNode.domNode.getChildNodes().getLength();
			for( int i = 0; i < lmtChildren; i++ ) {
				
			    if ( currNode.domNode.getChildNodes().item( i ).getNodeName().equals(
								    "OBSERVATION") ) {
				    System.out.println( "Found: " + 
							currNode.domNode.getChildNodes().item( i ).getNodeName() );
				    // When we find an OBSERVATION element, we take a look at it's
				    // elements in order to build column names for all unique instances of 
				    // monitored WebLogic Server MBeans.  
				    obsNode = new AdapterNode( currNode.domNode.getChildNodes().item( i ) );
				    if ( obsNode.domNode.hasChildNodes() ) {
					for ( int j = 0; j < obsNode.domNode.getChildNodes().getLength(); j++ ) {
					    if ( obsNode.domNode.getChildNodes().item(j).getNodeType() == ELEMENT_TYPE ) {
						if (
						    obsNode.domNode.getChildNodes().item(j).getNodeName().equals("ExecuteQueueRuntimeMBean") ) {
						    if (xml2t4.verboseStatus) {
							System.out.println( "   ExecuteQueueRuntimeMBean" );
						    }

						    pfxStr = new StringBuffer("ExecuteQueueRuntimeMBean:");
						    
						    
						    wrkNode = new AdapterNode(obsNode.domNode.getChildNodes().item(j));
						    for ( int k = 0; k < wrkNode.domNode.getChildNodes().getLength(); k++ ) {
							
							if (wrkNode.domNode.getChildNodes().item(k).getNodeType() == ELEMENT_TYPE &&
							    wrkNode.domNode.getChildNodes().item(k).getNodeName().equals("Name") ) {
							    if (xml2t4.verboseStatus) {
								System.out.println("      " + pfxStr + 
									       wrkNode.domNode.getChildNodes().item(k).getChildNodes().item(0).getNodeValue().trim() );
							    }
							    pfxStr.append(wrkNode.domNode.getChildNodes().item(k).getChildNodes().item(0).getNodeValue().trim() + ":" );
							    
							}
							if (wrkNode.domNode.getChildNodes().item(k).getNodeType() == ELEMENT_TYPE &&
							    !wrkNode.domNode.getChildNodes().item(k).getNodeName().equals("Name") ) {
							    if (xml2t4.verboseStatus) {
								System.out.println("      " + pfxStr + 
									       wrkNode.domNode.getChildNodes().item(k).getNodeName());
							    }
							    keyStr = new StringBuffer();
							    keyStr.append( pfxStr.toString() + wrkNode.domNode.getChildNodes().item(k).getNodeName());
							    if ( !xml2t4.hmNames.containsKey( keyStr.toString() ) ) { 
								xml2t4.hmNames.put( keyStr.toString(), "ColumnNameValue" );
							    }
							    
							}
							    
						    }
							
						} else if (obsNode.domNode.getChildNodes().item(j).getNodeName().equals("JDBCConnectionPoolRuntimeMBean")){
						    if (xml2t4.verboseStatus) {
							System.out.println("   JDBCConnectionPoolRuntimeMBean" );
						    }    
						    pfxStr = new StringBuffer("JDBCConnectionPoolRuntimeMBean:");
						    
						    
						    wrkNode = new AdapterNode(obsNode.domNode.getChildNodes().item(j));
						    for ( int k = 0; k < wrkNode.domNode.getChildNodes().getLength(); k++ ) {
							
							if (wrkNode.domNode.getChildNodes().item(k).getNodeType() == ELEMENT_TYPE &&
							    wrkNode.domNode.getChildNodes().item(k).getNodeName().equals("Name") ) {
							    if (xml2t4.verboseStatus) {
								System.out.println("      " + pfxStr + 
									       wrkNode.domNode.getChildNodes().item(k).getChildNodes().item(0).getNodeValue().trim() );
							    }
							    pfxStr.append(wrkNode.domNode.getChildNodes().item(k).getChildNodes().item(0).getNodeValue().trim() + ":" );
							}
							if (wrkNode.domNode.getChildNodes().item(k).getNodeType() == ELEMENT_TYPE &&
							    !wrkNode.domNode.getChildNodes().item(k).getNodeName().equals("Name") ) {
							    if (xml2t4.verboseStatus) {
								System.out.println("      " + pfxStr + 
									       wrkNode.domNode.getChildNodes().item(k).getNodeName());
							    }
							    keyStr = new StringBuffer();
							    keyStr.append( pfxStr + wrkNode.domNode.getChildNodes().item(k).getNodeName());
							    if ( !xml2t4.hmNames.containsKey( keyStr.toString() ) ) { 
								xml2t4.hmNames.put( keyStr.toString(), "ColumnNameValue" );
							    }
							    
							}
							    
						    }
						} else if (obsNode.domNode.getChildNodes().item(j).getNodeName().equals("JVMRuntimeMBean")){
						    if (xml2t4.verboseStatus) {
							System.out.println("   JVMRuntimeMBean" );
						    }
						    pfxStr = new StringBuffer("JVMRuntimeMBean:");
						    
						    
						    wrkNode = new AdapterNode(obsNode.domNode.getChildNodes().item(j));
						    for ( int k = 0; k < wrkNode.domNode.getChildNodes().getLength(); k++ ) {
							
							if (wrkNode.domNode.getChildNodes().item(k).getNodeType() == ELEMENT_TYPE ) {
							    if (xml2t4.verboseStatus) {
								System.out.println("      " + pfxStr + 
									       wrkNode.domNode.getChildNodes().item(k).getNodeName());
							    }
							    keyStr = new StringBuffer();
							    keyStr.append( pfxStr + wrkNode.domNode.getChildNodes().item(k).getNodeName());
							    if ( !xml2t4.hmNames.containsKey( keyStr.toString() ) ) { 
								xml2t4.hmNames.put( keyStr.toString(), "ColumnNameValue" );
							    }
							    
							}
							    
						    }
						} else if (obsNode.domNode.getChildNodes().item(j).getNodeName().equals("ServerMBean")){
						    if (xml2t4.verboseStatus) {
							System.out.println("   ServerMBean" );
						    }    
						    pfxStr = new StringBuffer("ServerMBean:");
						    
						    
						    wrkNode = new AdapterNode(obsNode.domNode.getChildNodes().item(j));
						    for ( int k = 0; k < wrkNode.domNode.getChildNodes().getLength(); k++ ) {
							
							if (wrkNode.domNode.getChildNodes().item(k).getNodeType() == ELEMENT_TYPE &&
							    wrkNode.domNode.getChildNodes().item(k).getNodeName().equals("Name") ) {
							    if (xml2t4.verboseStatus) {
								System.out.println("      " + pfxStr + 
									       wrkNode.domNode.getChildNodes().item(k).getChildNodes().item(0).getNodeValue().trim() );
							    }
							    pfxStr.append(wrkNode.domNode.getChildNodes().item(k).getChildNodes().item(0).getNodeValue().trim() + ":" );
							}
							if (wrkNode.domNode.getChildNodes().item(k).getNodeType() == ELEMENT_TYPE &&
							    !wrkNode.domNode.getChildNodes().item(k).getNodeName().equals("Name") ) {
							    if (xml2t4.verboseStatus) {
								System.out.println("      " + pfxStr + 
									       wrkNode.domNode.getChildNodes().item(k).getNodeName());
							    }
							    keyStr = new StringBuffer();
							    keyStr.append( pfxStr + wrkNode.domNode.getChildNodes().item(k).getNodeName());
							    if ( !xml2t4.hmNames.containsKey( keyStr.toString() ) ) { 
								xml2t4.hmNames.put( keyStr.toString(), "ColumnNameValue" );
							    }
							    keyStr = new StringBuffer();
							    keyStr.append( pfxStr + wrkNode.domNode.getChildNodes().item(k).getNodeName());
							    if ( !xml2t4.hmNames.containsKey( keyStr.toString() ) ) { 
								xml2t4.hmNames.put( keyStr.toString(), "ColumnNameValue" );
							    }
							    
							}
							    
						    }

						}
					    } 
					}
				    } else {
					System.out.println("OBSERVATION element without CHILD elements!");
					System.exit( -1 );
				    }
			    }
			}
			
			endOfObservations = true;
		    } else {
			    System.out.println("ERROR: WEBLOGICMONITORSESSION has NO CHILDREN!");
			    System.exit( -1 );
		    }
		}
		    

		if (xml2t4.verboseStatus) {
		    System.out.println( "Size of HashMap is: " + xml2t4.hmNames.size() );
		    HashSet setOfKeys = new HashSet( xml2t4.hmNames.keySet() );
		    Iterator iterKeys = setOfKeys.iterator();
		    while( iterKeys.hasNext() ) {
			String strKey = new String( iterKeys.next().toString() );
			String strVal = new String( xml2t4.hmNames.get( strKey ).toString() );
			System.out.println( "Key is: " + strKey + " Content is: " + strVal );
		    }
		}




				
       	} catch (SAXException sxe) {
       // Error generated during parsing
       Exception  x = sxe;
       if (sxe.getException() != null)
           x = sxe.getException();
       x.printStackTrace();

    	} catch (ParserConfigurationException pce) {
       // Parser with specified options can't be built
       pce.printStackTrace();
    	} catch (IOException ioe) {
       // I/O error
       ioe.printStackTrace();
    }

	xml2t4.listAll( xml2t4.outTimeLinePrint );




}// main

    // An array of names for DOM node-types
    // (Array indexes = nodeType() values.)
    static final String[] typeName = {
        "none",
        "Element",
        "Attr",
        "Text",
        "CDATA",
        "EntityRef",
        "Entity",
        "ProcInstr",
        "Comment",
        "Document",
        "DocType",
        "DocFragment",
        "Notation",
    };

    static final int ELEMENT_TYPE =   1;
    static final int ATTR_TYPE =      2;
    static final int TEXT_TYPE =      3;
    static final int CDATA_TYPE =     4;
    static final int ENTITYREF_TYPE = 5;
    static final int ENTITY_TYPE =    6;
    static final int PROCINSTR_TYPE = 7;
    static final int COMMENT_TYPE =   8;
    static final int DOCUMENT_TYPE =  9;
    static final int DOCTYPE_TYPE =  10;
    static final int DOCFRAG_TYPE =  11;
    static final int NOTATION_TYPE = 12; 

    // The list of elements to display in the tree
    // Note that this should be developed from parsing DTD or XML Schema,
    // not hard coded.
    static String[] treeElementNames = {
        "WEBLOGICMONITORSESSION",
        "STARTSTATE",
	"OBSERVATION",
			
 
			
    };
    // The list of elements that will be examined in order to get the subelements
    // associated with them, so that column names can be built for output file.
    static String[] elemNames = {
        "ExecuteQueueRuntimeMBean",
        "JVMRuntimeMBean",
	"JDBCConnectionPoolRuntime",
	"ServerMBean",
    };
    boolean treeElement(String elementName) {
      for (int i=0; i<treeElementNames.length; i++) {
        if ( elementName.equals(treeElementNames[i]) ) return true;
      }
      return false;
    } 
    boolean elemColumnName(String elementName) {

      for (int i=0; i < elemNames.length; i++) {
        if ( elementName.equals( elemNames[i] ) ) return true;
      }
      return false;
    } 

    public void listAll( PrintWriter pw ) {

	// First order of biz is to put out T4 like header lines

	String nodeName = new String( "TESTME" );
	String firstDate = new String( "12-DEC-2003" );
	String firstTime = new String( "16:26:00" );

	StringBuffer sbuf = new StringBuffer();
	sbuf.append( nodeName );
	sbuf.append( "," );
	sbuf.append( "T4EXTR Version: T4 Version : Vx.y-zzz ... 15-Sep-2003,OpenVMS Vx.y-z,T4EXTR Id : Tx.y-z,NET_MON Id : Tx.y-z," );
	sbuf.append( "TCP_MON Id : Tx.y-z,XFC_MON Id : Tx.y-z,LCK73_MON Id : Tx.y-z,ACLG Id : Tx.y-z,APRC Id : Tx.y-z,TCP Version: Vx.y-z," );
	sbuf.append( "File Format Type: x, Analyzer node: " );
	sbuf.append( nodeName );
	sbuf.append( ", Monitor Comment: **TBD**, MpCpus: ?, Cpu Conf: ????????,Sampling Interval: ?? seconds, Decnet Version: ????????," );
	sbuf.append( "Command: Proc Counts by Interval, ," );
	sbuf.append( nodeName );
	sbuf.append( "," );
	for( int i = 0; i < 17; i++ ) {
	    sbuf.append( nodeName );
	    sbuf.append( "," );
	}
	sbuf.append( nodeName );
	pw.println( sbuf.toString() );
	sbuf.setLength( 0 );
	for( int j = 0; j < 20; j++ ) {
	    sbuf.append( firstDate );
	    sbuf.append( "," );
	}
	sbuf.append( firstDate );
	pw.println( sbuf.toString() );
	sbuf.setLength( 0 );
	for( int k = 0; k < 20; k++ ) {
	    sbuf.append( firstTime );
	    sbuf.append( "," );
	}
	sbuf.append( firstTime );
	pw.println( sbuf.toString() );
    
	sbuf.setLength( 0 );
	HashSet setOfKeys = new HashSet( hmNames.keySet() );
	ArrayList arrayOfKeys = new ArrayList();
	Iterator iterKeys = setOfKeys.iterator();
	while( iterKeys.hasNext() ) {
	    String strKey = new String( iterKeys.next().toString() );
	    arrayOfKeys.add( strKey );
	}
	Collections.sort( arrayOfKeys );
	sbuf.setLength( 0 );
	sbuf.append( "[WEBLOGICMONITOR]Sample Time," );
	for( int m = 0; m < arrayOfKeys.size(); m++ ) {
	    String strVal = new String( hmNames.get( arrayOfKeys.get( m ) ).toString() );
	    String wrkStr = new String( "[" +  arrayOfKeys.get( m ).toString() + "]" );
	    sbuf.append( wrkStr + "," );
 	}
				      
    
	sbuf.deleteCharAt( sbuf.lastIndexOf( "," ) );
	pw.println( sbuf.toString() );

	// Go through document tree to create column names for T4 output file
	AdapterNode rootNode = new AdapterNode( document );
	AdapterNode currNode = new AdapterNode( document );
	AdapterNode obsNode = new AdapterNode( document );
	AdapterNode wrkNode = new AdapterNode( document );
	boolean endOfObservations = false;
	String nameStr = new String( "***Error***" );
	String mbeanStr = new String( "***Error***" );
	String valStr = new String( "***Error***" );
	// From the root node, find the WEBLOGICMONITORSESSION element
	boolean elementFnd = false;
	while( !elementFnd ) {
			
	    if ( currNode.domNode.hasChildNodes() ) {

		for( int i = 0; i < currNode.domNode.getChildNodes().getLength() && !elementFnd; i++ ) {
				
		    if ( currNode.domNode.getChildNodes().item( i ).getNodeName().equals(
								    "WEBLOGICMONITORSESSION") ) {
			    elementFnd = true;
			    currNode = new AdapterNode( currNode.domNode.getChildNodes().item( i ) );
		    }
			       
		} // endfor

	    } else {
		System.out.println("ERROR: WEBLOGICMONITORSESSION element not found!");
		System.exit( -1 );
	    }
	} 
	// Assumption is that WEBLOGICMONITORSESSION element found
	// Therefore, we work through the OBSERVATION elements in order to build
	// the detail lines
	if ( currNode.domNode.hasChildNodes() ) {

	    int lmtChildren = currNode.domNode.getChildNodes().getLength();

	    for( int i = 0; i < lmtChildren; i++ ) {

		sbuf.setLength(0);
				
		if ( currNode.domNode.getChildNodes().item( i ).getNodeName().equals(
								    "OBSERVATION") ) {
		    // When we find an OBSERVATION element, we first retrieve the value of TIMESTAMP  
		    obsNode = new AdapterNode( currNode.domNode.getChildNodes().item( i ) );
		    String timestamp = new String( obsNode.getChildValue( "TIMESTAMP" ) );
		    
		    
		    sbuf.append( new String( timestamp ).trim() );
		    sbuf.append( "," );

		    for( int m = 0; m < arrayOfKeys.size(); m++ ) {
			String wrkStr = new String( arrayOfKeys.get( m ).toString() );
			StringTokenizer st1 = new StringTokenizer( wrkStr, ":]", false );
			mbeanStr = new String( st1.nextToken() );
			if ( mbeanStr.equals( "ExecuteQueueRuntimeMBean") ||
			     mbeanStr.equals( "ServerMBean") ||
			     mbeanStr.equals( "JDBCConnectionPoolRuntimeMBean" ) ) {

			    nameStr = new String( st1.nextToken() );
			    valStr = new String( st1.nextToken() );

			    sbuf.append( new String(obsNode.getObsValue( mbeanStr, nameStr, valStr )).trim() );
			    sbuf.append( "," );
			} else {
			    nameStr = new String( st1.nextToken() );
			    sbuf.append( new String( obsNode.getObsValue( mbeanStr, nameStr ) ).trim() );
			    sbuf.append( "," );
			}
			    

			    
			    
			    
			    
		    }
			sbuf.deleteCharAt( sbuf.lastIndexOf( "," ) );
			pw.println( sbuf.toString() );


		}
	    } 

	} else {
	    System.out.println("OBSERVATION element without CHILD elements!");
	    System.exit( -1 );
	}
		

    }

}// XML2T4CSV
