$! ======================================================================
$! Wlstlt.Com          Version 1.00        February 2004
$!
$! WebLogic Server TimeLine Tool (WLSTLT)
$!
$! Purpose:
$!
$!   To offer a WebLogic Server data collector plugin for the HP T4 data
$!   collector tool.
$!
$!   This script builds a DCL script per WebLogic Server Instance to be
$!   monitored and starts the WLSTLT collector.
$!
$!   The Instance to be monitored must :
$!
$!   o be WebLogic Server 8.1 SP1 or higher
$!
$!   o be registered in the file t4$sys:WlstltTab.Ora
$!     See WLSTLT.txt for details of this file format 
$!
$!   o have the automonitor flag in t4$sys:WlstltTab.Ora set to 'Y'
$!
$! Usage:
$!
$!   Mainly called by the HP T4 Data collecter but can be called 
$!   with a single parameter of CHECK to test/confirm actions.
$!   When called with the CHECK parameter, the data collecter will
$!   not be started.
$!
$!   Parameters passed are :
$!
$!   P1 - Batch Queue Name (Or CHECK to test the code)
$!   P2 - Start Time
$!   P3 - End Time
$!   P4 - Sampling interval in seconds (default is 60)
$!   P5 - date/time string for output file name
$!   P6 - Work directory (ie, directory to store data)
$!   
$!
$! OUTPUT
$!
$!   Output is sent to the T4 collector output directory as specified
$!   by the user. Note that the output in XML format, for later 
$!   processing.
$!
$!
$! ======================================================================
$!
$!      Modification Log
$!
$!      V1.00 Pat McConnell           18 February 2004
$!
$!  Get node name
$!
$ Node = F$Edit(F$GetSyi("NODENAME"),"UPCASE")
$ wls_cnt = 0
$!
$ If P1 .Eqs. "" Then exit
$ If P1 .Nes. "CHECK" 
$ Then
$    Check_Only = "FALSE"
$    If P2 .Eqs. "" Then Exit
$    If P3 .Eqs. "" Then Exit
$    If P4 .Eqs. "" Then Exit
$    If P5 .Eqs. "" Then Exit
$    If P6 .Eqs. "" Then Exit
$ Else
$    Check_Only = "TRUE"
$ EndIf
$!
$!
$!
$! Find WebLogic Server instances to be monitored.
$! Uses the t4$sys:WlstltTab.Wls file as the driver
$!
$ Say = "Write Sys$OutPut"
$ Close/Nolog WLSInsts
$ Open/Read/Error=Nothere WLSInsts t4$sys:WlstltTab.Wls
$!
$ Say " Scanning t4$sys:WlstltTab.wls for WebLogic Server Instances to monitor..."
$!
$PROCESS_INSTANCES:
$!
$ Read/Error=Read_Err/End=Finished WLSInsts Record
$!
$ If f$Extract(0,1,Record) .eqs. "$" then goto Process_instances
$!
$ WLSName      = F$Element(0,",",Record)
$ WLSURL     = F$Element(1,",",Record)
$ WLSPort    = F$Element(2,",",Record)
$ WLSAdmin    = F$Element(3,",",Record)
$ WLSAdminPswd = F$Element(4,",",Record)
$ Monit = F$Element(5,",",Record)
$ BEAHome = F$Element(6,",",Record)
$!
$!
$!  Have we been asked to monitor it
$!
$ If Monit .Eqs. "N"
$ Then
$    Say " Skipping WebLogic Server instance ''WLSName' - it is set to NO monitor"
$    Goto process_instances
$ EndIf
$!
$!  SHOULD DO SOME CHECKING FOR JAVA AT THIS POINT -- BUT LET US POSIT
$!  THE EXISTENCE OF SAID JAVA ON THE SYSTEM FOR NOW.  WE ARE ALSO 
$!  ASSUMING THE EXISTENCE OF A BEA WEBLOGIC SERVER 8.1 SP1 OR HIGHER.
$!
$ @'BEAHome'setEnvVMS.com
$ DEF /JOB JAVA$CLASSPATH wls_home:[.lib]weblogic.jar,t4$sys
$!
$!  Increment instance counter
$!
$ wls_cnt = wls_cnt + 1
$!
$!
$!
$!  Confirm the file exists
$!
$!
$       If .Not. Check_Only
$       Then
$          Outpt = "t4$sys:Wlstlt_''WLSName'.Com"
$          Close/Nolog DclScr
$          Open/Write DclScr 'Outpt'
$          Write DclScr "$ Set NoOn"
$          Write DclScr "$ set def t4$sys"
$          Write DclScr "$ Set Process/Name=" + """" + "WLSMon" + """"
$          Write DclScr "$ @''BEAHome'" + "setEnvVMS.com"
$ sho log wls*
$          Write DclScr -
 "$ def/job java$classpath wl_home:[server.lib]weblogic.jar,t4$sys:[serverExQInfo]"
$ sho log java$*
$          Write DclScr -
    "$ java serverExQInfo -start " + """" + "''P2'" + """" + " -end " + -
    """" + "''P3'" + """" + -
    " -interval " + "''P4'" + " -url " + "''WLSURL'" + " -server " + -
    "''WLSName'" + " -user " + "''WLSAdmin'" + " -password " + -
    "''WLSAdminPswd'" + " -port " + "''WLSPort'" + " -ofile " + -
    "''Node'_''WLSName'.xml" 
$          Close/Nolog DclScr
$       EndIf
$!
$ Goto Process_Instances
$!
$FINISHED:
$!
$ If wls_cnt .gt. 0
$ Then
$    Cnt = 0
$START_THEM: 
$    Cnt = Cnt + 1
$!    
$    If Check_Only
$    Then
$       Say " Would have started WLSTLT for WebLogic Server instance ''WLSName'"
$    Else
$!
$!  Note that we use Batch. 
$!
$       Submit/Notify/Keep/NoPrint/Log=t4$sys:WLSTLT_'Node'_'WLSName' -
                t4$sys:Wlstlt_'WLSName.Com; -
                /Queue='P1 -
                /Name="''P7'_''WLSName'"
$    EndIf
$    If wls_cnt - Cnt .Gt. 0 Then Goto Start_Them
$ EndIf
$!
$! Done
$!
$ Goto The_End
$!
$NOTHERE:
$ Say " "
$ Say " The t4$sys:WlstltTab.wls file can not be located"
$ Say " "
$ Say " Execute t4$sys:Wlstlt_Find_WLSNames.Com and review the output"
$ Say " "
$ Goto The_End
$!
$READ_ERR:
$ Say " "
$ Say " Error reading file t4$sys:WlstltTab.wls"
$ Say " "
$ Goto The_End
$!
$THE_END:
$!
$ Close/Nolog WLSInsts
$!
$ Exit
